package cKonto;

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# QualityClick
# Copyright  15.06.2013 - NetSlave GmbH, www.netslave.de
#
# Usage:
# use cKonto;
#  cKonto->check ( kontonummer => "$kto", bankleitzahl => "$blz" );
#  cKonto->check ( iban => "$iban" );
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
use strict;
use Carp;
use LWP::UserAgent;

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

our $cKonto_url = 'https://www.ckonto.de/webservice.cgi';
my  $cKonto_key = ''; #!# IHR LIZENZSCHLÜSSEL
my  $timeout    = 90;

sub kontocheck {
   my $self = shift;
   my ($kto,$blz) = @_;
   __PACKAGE__->check ( kontonummer => "$kto", bankleitzahl => "$blz" );
}

sub ibancheck {
   my $self = shift;
   my ($iban) = @_;
   __PACKAGE__->check ( iban => "$iban" );
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub check {
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   my $self = shift;
   my %args = @_;
  
   my ($kto,$blz,$iban,$url);

   if (exists $args{kontonummer} && exists $args{bankleitzahl}) {
      ($kto = $args{kontonummer}) =~s/\D//gi;
      ($blz = $args{bankleitzahl}) =~s/\D//gi;
   }
   elsif (exists $args{iban} ) {
      ($iban = $args{iban}) =~ s/[^A-Z0-9]//gi;
   }

   unless (($kto && $blz) || $iban) {
      return ( 9, 'System-Fehler, ungueltige Parameter' );
   }

   $url  = "$cKonto_url" . q{?}; 
   if ($iban) { $url .= "iban=$iban"; }
   else       { $url .= "kontonummer=$kto&bankleitzahl=$blz"; }

   $url .= '&key=' . $cKonto_key;
     
   my $ua = new LWP::UserAgent;
   $ua->timeout($timeout);
   $ua->agent('Mozilla/5.5 (compatible; MSIE 5.5; Windows NT 5.1)');

   my $request = HTTP::Request->new('GET');
   $request->url($url);

   my $response = '';

   eval {
      local $SIG{ALRM} = sub { die "Alarm" };
      alarm($timeout);
      $response = $ua->request($request);
      alarm($timeout);
   };

   if ( $@ and $@ !~ /Alarm/ ) {
      print STDERR "cKonto.pm: $url -- Keine Antwort: Zeitueberschreitung\n";
      return ( 9, 'Zeitueberschreitung - Die Antwort vom Server dauerte laenger als 90 Sekunden' );
   }    #///

   my $html = $response->content;

   return _parse_response(($iban ? 1:0), $html);
}#///


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Helpers
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
my %cKonto_msg_kto = (
  0 => 'Die Kontonummer ist ungültig, sie kann von diesem Institut nicht vergeben werden',
  1 => 'Die Kontonummer ist gültig, sie kann von diesem Institut vergeben werden',
  2 => 'Fehler bei der Eingabe der Bankleitzahl (Sie ist entweder nicht 8-stellig oder enthält Buchstaben)',
  3 => 'Fehler bei der Eingabe der Kontonummer (Sie ist entweder zu lang oder enthält Buchstaben',
  4 => 'Eingabefehler bei der Kontonummer und Bankleitzahl (siehe 2 und 3)',
  5 => 'Genereller Eingabefehler des Übergabeparameters - enthält evtl. Leerzeichen',
  6 => 'Fehler im Format des Übergabeparameters',
  7 => 'Die Bankleitzahl wurde in der Datenbank nicht gefunden (existiert nicht)',
  8 => 'Die von der Bank verwendete Prüfmethode ist im Demonstrations-Modus nicht verfügbar',
  9 => 'Die Kontonummer kann nicht geprüft werden, da die Bank entweder keine Prüfziffern verwendet oder es sich um eine spezielle Kontonummer handelt',
);

my %cKonto_msg_iban    = %cKonto_msg_kto;
   $cKonto_msg_iban{2} = 'Eingabefehler BLZ';
   $cKonto_msg_iban{3} = 'Eingabefehler KNR';
   $cKonto_msg_iban{4} = 'Eingabefehler IBAN';

sub _parse_response {
  my $iban_modus = shift;
  my $html = shift;
  my ($status,$msg);

  if ($html =~ m{<ckonto_check>(.*)</ckonto_check>}is) {
     $html = $1;
     if ($html =~ m{<status>(.*)</status>}is) {
        $status = $1;
     }
     
  }
  unless (defined $status) {
     return (9, 'Systemfehler'); 
  }
  
  $msg = $iban_modus ? $cKonto_msg_iban{$status} : $cKonto_msg_kto{$status} ;

  return ($status, $msg); 
}

1;    # This is a perl module so we must return true to perl
