#! /usr/bin/env python
import pycurl
import re
import StringIO
import sys
import urllib
import xml.dom.minidom


class Error(Exception):
	"""Base class for exceptions in this module"""
	pass

class InvalidValueException(Error):
	def __init__(self,message):
		self.message	= message

class InvalidKeyException(Error):
	def __init__(self,key):
		self.message	= 'You provided a wrong license key ( %s )!' % key

class CurlConnectionException(Error):
	def __init__(self,status):
		self.message	= 'curl returned error %s' % status

class Connection:
	""" Class for checking german bank account numbers and BLZs against https://www.ckonto.de/webservice.cgi
	You need to have a valid license key for that"""

	def __init__ (self,key='demo'):
		self.key	= key
	
	def check(self,blz=None,kto=None):
		""" The check function does the actual check of the account """

		# Both the BLZ and the Kto numbers are checked for beeing syntactically correct as the curl request takes some time
		blz_pattern	= re.compile('^\d{8}$')
		if (blz_pattern.match(blz) == None):
			raise InvalidValueException('Invalid value in BLZ')

		kto_pattern	= re.compile('^\d{1,10}$')
		if (kto_pattern.match(kto) == None):
			raise InvalidValueException('Invalid value in Kto')

		# Create an StringIO object for the pycurl.WRITEFUNCTION callback
		xmldata = StringIO.StringIO()
		
		# urlencode the necessary parameters for the request
		params = urllib.urlencode({'key':self.key,'kontonummer':kto,'bankleitzahl':blz})

		curl = pycurl.Curl()
		curl.setopt(pycurl.URL,'https://www.ckonto.de/webservice.cgi')
		curl.setopt(pycurl.WRITEFUNCTION, xmldata.write )
		curl.setopt(pycurl.POST,1)
		curl.setopt(pycurl.POSTFIELDS,params)
		curl.perform()
		curl_status = curl.getinfo(pycurl.HTTP_CODE)
		if ( not curl_status == 200):
			raise CurlConnectionException(curl_status)

		curl.close()
		# Parse the raw XML data
		xmlobj	= xml.dom.minidom.parseString(xmldata.getvalue())
		
		# initialize the result dictionary for global use
		result	= {}

		for element in ('bank','blz','kto','location','status','zip'):
			result[element]	= xmlobj.getElementsByTagName(element)[0].firstChild.nodeValue

		return result
